/* GPL2 */
#include <linux/module.h>

#include "dvb_frontend.h"
#include "va9902a.h"

MODULE_DESCRIPTION( "VA1A5JZ9902A Frontend" );
MODULE_LICENSE( "GPL" );
MODULE_VERSION( "0.1" );

static int debug = 1;
#define dbginfo(format, arg...) do{ if ( debug ) printk( KERN_DEBUG "va9902a: " format "\n" , ## arg); }while(0)


struct va9902a_state{
    struct i2c_adapter* i2c;
    struct dvb_frontend frontend;
};


static int va9902a_read_status(struct dvb_frontend* fe, fe_status_t* status)
{
    // 適当に決めたので正しいのか不明 (これ以上削ると動かない)
    *status = FE_HAS_LOCK | FE_HAS_CARRIER;

    return 0;
}


// 周波数変更 & 出力開始
static int va9902a_set_frontend( struct dvb_frontend* fe, struct dvb_frontend_parameters *param )
{
    int ret;
    u8 buf[ 8 ];
    struct i2c_msg msg;
    struct va9902a_state* state = fe->demodulator_priv;
    int freq = param->frequency / 1000;

    dbginfo( "set_frontend freq = %d khz", freq );

    buf[ 0 ] = ( freq >> 16 ) & 0xff;
    buf[ 1 ] = ( freq >> 8 ) & 0xff;
    buf[ 2 ] = freq & 0xff;

    msg.flags = 0;
    msg.buf = buf;
    msg.len = 3;

    // i2c経由でdus01に出力を命令
    ret = i2c_transfer( state->i2c, &msg, 1 );
    if( ret != 1 ){
        dbginfo( "i2c_transfer failed" );
        return -EREMOTEIO;
    }

    return 0;
}


static void va9902a_release(struct dvb_frontend* fe)
{
    struct va9902a_state* state = fe->demodulator_priv;

    dbginfo( "release" );

    kfree( state );
}


// 適当に決めたので正しいのか不明
static struct dvb_frontend_ops va9902a_ops = {

    .info = {
        .name = "VA9902A",
        .type = FE_OFDM,
        .frequency_min = 0,
        .frequency_max = 862000000,

        .caps = FE_CAN_FEC_1_2 | FE_CAN_FEC_2_3 | FE_CAN_FEC_3_4 |
        FE_CAN_FEC_4_5 | FE_CAN_FEC_5_6 | FE_CAN_FEC_6_7 |
        FE_CAN_FEC_7_8 | FE_CAN_FEC_8_9 | FE_CAN_FEC_AUTO |
        FE_CAN_QAM_16 | FE_CAN_QAM_64 | FE_CAN_QAM_AUTO,
    },

    .read_status = va9902a_read_status,
    .set_frontend = va9902a_set_frontend,
    .release = va9902a_release,
};


struct dvb_frontend* va9902a_attach( struct i2c_adapter* i2c )
{
    struct va9902a_state* state = NULL;

    dbginfo( "attach" );

    state = kmalloc( sizeof( struct va9902a_state ), GFP_KERNEL );
    if( state == NULL ) return NULL;

    memcpy( &state->frontend.ops, &va9902a_ops, sizeof( struct dvb_frontend_ops ) );
    state->frontend.demodulator_priv = state;
    state->i2c = i2c;

    return &state->frontend;
}
EXPORT_SYMBOL( va9902a_attach );
